[Asterisk-code-review] res rtp asterisk.c: Add "seqno" strictrtp option (asterisk[master])
George Joseph
asteriskteam at digium.com
Fri Sep 28 07:27:25 CDT 2018
George Joseph has submitted this change and it was merged. ( https://gerrit.asterisk.org/10249 )
Change subject: res_rtp_asterisk.c: Add "seqno" strictrtp option
......................................................................
res_rtp_asterisk.c: Add "seqno" strictrtp option
When networks experience disruptions, there can be large gaps of time
between receiving packets. When strictrtp is enabled, this created
issues where a flood of packets could come in and be seen as an attack.
Another option - seqno - has been added to the strictrtp option that
ignores the time interval and goes strictly by sequence number for
validity.
Change-Id: I8a42b8d193673899c8fc22fe7f98ea87df89be71
---
M CHANGES
M configs/samples/rtp.conf.sample
M res/res_rtp_asterisk.c
3 files changed, 65 insertions(+), 23 deletions(-)
Approvals:
Richard Mudgett: Looks good to me, but someone else must approve
Joshua Colp: Looks good to me, but someone else must approve
Kevin Harwell: Looks good to me, approved
George Joseph: Approved for Submit
diff --git a/CHANGES b/CHANGES
index 26748f7..fef7212 100644
--- a/CHANGES
+++ b/CHANGES
@@ -9,6 +9,17 @@
==============================================================================
------------------------------------------------------------------------------
+--- Functionality changes from Asterisk 16.0.0 to Asterisk 16.1.0 ------------
+------------------------------------------------------------------------------
+
+res_rtp_asterisk
+------------------
+ * The existing strictrtp option in rtp.conf has a new choice availabe, called
+ 'seqno', which behaves the same way as setting strictrtp to 'yes', but will
+ ignore the time interval during learning so that bursts of packets can still
+ trigger learning our source.
+
+------------------------------------------------------------------------------
--- Functionality changes from Asterisk 15 to Asterisk 16 --------------------
------------------------------------------------------------------------------
diff --git a/configs/samples/rtp.conf.sample b/configs/samples/rtp.conf.sample
index de9d590..26a70d2 100644
--- a/configs/samples/rtp.conf.sample
+++ b/configs/samples/rtp.conf.sample
@@ -31,6 +31,10 @@
; seconds after starting learning mode. Once learning mode completes the
; current stream is locked in and cannot change until the next
; renegotiation.
+; Valid options are "no" to disable strictrtp, "yes" to enable strictrtp,
+; and "seqno", which does the same thing as strictrtp=yes, but only checks
+; to make sure the sequence number is correct rather than checking the time
+; interval as well.
; This option is enabled by default.
; strictrtp=yes
;
diff --git a/res/res_rtp_asterisk.c b/res/res_rtp_asterisk.c
index 192840c..5f7cd9f 100644
--- a/res/res_rtp_asterisk.c
+++ b/res/res_rtp_asterisk.c
@@ -157,6 +157,12 @@
STRICT_RTP_CLOSED, /*! Drop all RTP packets not coming from source that was learned */
};
+enum strict_rtp_mode {
+ STRICT_RTP_NO = 0, /*! Don't adhere to any strict RTP rules */
+ STRICT_RTP_YES, /*! Strict RTP that restricts packets based on time and sequence number */
+ STRICT_RTP_SEQNO, /*! Strict RTP that restricts packets based on sequence number */
+};
+
/*!
* \brief Strict RTP learning timeout time in milliseconds
*
@@ -166,7 +172,7 @@
*/
#define STRICT_RTP_LEARN_TIMEOUT 5000
-#define DEFAULT_STRICT_RTP -1 /*!< Enabled */
+#define DEFAULT_STRICT_RTP STRICT_RTP_YES /*!< Enabled by default */
#define DEFAULT_ICESUPPORT 1
extern struct ast_srtp_res *res_srtp;
@@ -3154,28 +3160,31 @@
info->received = ast_tvnow();
}
- switch (info->stream_type) {
- case AST_MEDIA_TYPE_UNKNOWN:
- case AST_MEDIA_TYPE_AUDIO:
- /*
- * Protect against packet floods by checking that we
- * received the packet sequence in at least the minimum
- * allowed time.
- */
- if (ast_tvzero(info->received)) {
- info->received = ast_tvnow();
- } else if (!info->packets
- && ast_tvdiff_ms(ast_tvnow(), info->received) < learning_min_duration) {
- /* Packet flood; reset */
- info->packets = learning_min_sequential - 1;
- info->received = ast_tvnow();
+ /* Only check time if strictrtp is set to yes. Otherwise, we only needed to check seqno */
+ if (strictrtp == STRICT_RTP_YES) {
+ switch (info->stream_type) {
+ case AST_MEDIA_TYPE_UNKNOWN:
+ case AST_MEDIA_TYPE_AUDIO:
+ /*
+ * Protect against packet floods by checking that we
+ * received the packet sequence in at least the minimum
+ * allowed time.
+ */
+ if (ast_tvzero(info->received)) {
+ info->received = ast_tvnow();
+ } else if (!info->packets
+ && ast_tvdiff_ms(ast_tvnow(), info->received) < learning_min_duration) {
+ /* Packet flood; reset */
+ info->packets = learning_min_sequential - 1;
+ info->received = ast_tvnow();
+ }
+ break;
+ case AST_MEDIA_TYPE_VIDEO:
+ case AST_MEDIA_TYPE_IMAGE:
+ case AST_MEDIA_TYPE_TEXT:
+ case AST_MEDIA_TYPE_END:
+ break;
}
- break;
- case AST_MEDIA_TYPE_VIDEO:
- case AST_MEDIA_TYPE_IMAGE:
- case AST_MEDIA_TYPE_TEXT:
- case AST_MEDIA_TYPE_END:
- break;
}
info->max_seq = seq;
@@ -6736,6 +6745,8 @@
&& STRICT_RTP_LEARN_TIMEOUT < ast_tvdiff_ms(ast_tvnow(), rtp->rtp_source_learn.start)) {
ast_verb(4, "%p -- Strict RTP learning complete - Locking on source address %s\n",
rtp, ast_sockaddr_stringify(&rtp->strict_rtp_address));
+ ast_test_suite_event_notify("STRICT_RTP_LEARN", "Source: %s",
+ ast_sockaddr_stringify(&rtp->strict_rtp_address));
rtp->strict_rtp_state = STRICT_RTP_CLOSED;
} else {
struct ast_sockaddr target_address;
@@ -6822,6 +6833,16 @@
}
ast_debug(1, "%p -- Received RTP packet from %s, dropping due to strict RTP protection.\n",
rtp, ast_sockaddr_stringify(&addr));
+#ifdef TEST_FRAMEWORK
+ {
+ static int strict_rtp_test_event = 1;
+ if (strict_rtp_test_event) {
+ ast_test_suite_event_notify("STRICT_RTP_CLOSED", "Source: %s",
+ ast_sockaddr_stringify(&addr));
+ strict_rtp_test_event = 0; /* Only run this event once to prevent possible spam */
+ }
+ }
+#endif
return &ast_null_frame;
case STRICT_RTP_OPEN:
break;
@@ -8110,7 +8131,13 @@
};
}
if ((s = ast_variable_retrieve(cfg, "general", "strictrtp"))) {
- strictrtp = ast_true(s);
+ if (ast_true(s)) {
+ strictrtp = STRICT_RTP_YES;
+ } else if (!strcasecmp(s, "seqno")) {
+ strictrtp = STRICT_RTP_SEQNO;
+ } else {
+ strictrtp = STRICT_RTP_NO;
+ }
}
if ((s = ast_variable_retrieve(cfg, "general", "probation"))) {
if ((sscanf(s, "%d", &learning_min_sequential) != 1) || learning_min_sequential <= 1) {
--
To view, visit https://gerrit.asterisk.org/10249
To unsubscribe, or for help writing mail filters, visit https://gerrit.asterisk.org/settings
Gerrit-Project: asterisk
Gerrit-Branch: master
Gerrit-MessageType: merged
Gerrit-Change-Id: I8a42b8d193673899c8fc22fe7f98ea87df89be71
Gerrit-Change-Number: 10249
Gerrit-PatchSet: 6
Gerrit-Owner: Benjamin Keith Ford <bford at digium.com>
Gerrit-Reviewer: George Joseph <gjoseph at digium.com>
Gerrit-Reviewer: Jenkins2 (1000185)
Gerrit-Reviewer: Joshua Colp <jcolp at digium.com>
Gerrit-Reviewer: Kevin Harwell <kharwell at digium.com>
Gerrit-Reviewer: Richard Mudgett <rmudgett at digium.com>
-------------- next part --------------
An HTML attachment was scrubbed...
URL: <http://lists.digium.com/pipermail/asterisk-code-review/attachments/20180928/540dba57/attachment.html>
More information about the asterisk-code-review
mailing list